# coding: utf-8

"""
    Conversations Inbox & Messages

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v3
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import List, Optional
from typing_extensions import Annotated
from hubspot.conversations.conversations.models.collection_response_public_message_forward_paging import CollectionResponsePublicMessageForwardPaging
from hubspot.conversations.conversations.models.collection_response_public_thread_forward_paging import CollectionResponsePublicThreadForwardPaging
from hubspot.conversations.conversations.models.public_message import PublicMessage
from hubspot.conversations.conversations.models.public_message_content import PublicMessageContent
from hubspot.conversations.conversations.models.public_message_egg import PublicMessageEgg
from hubspot.conversations.conversations.models.public_thread import PublicThread
from hubspot.conversations.conversations.models.public_thread_update_request import PublicThreadUpdateRequest

from hubspot.conversations.conversations.api_client import ApiClient, RequestSerialized
from hubspot.conversations.conversations.api_response import ApiResponse
from hubspot.conversations.conversations.rest import RESTResponseType


class ThreadsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def archive_thread_by_id(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Archives a thread.

        Archives a single thread. The thread will be permanently deleted 30 days after placed in an archived state.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._archive_thread_by_id_serialize(
            thread_id=thread_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def archive_thread_by_id_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Archives a thread.

        Archives a single thread. The thread will be permanently deleted 30 days after placed in an archived state.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._archive_thread_by_id_serialize(
            thread_id=thread_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def archive_thread_by_id_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Archives a thread.

        Archives a single thread. The thread will be permanently deleted 30 days after placed in an archived state.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._archive_thread_by_id_serialize(
            thread_id=thread_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _archive_thread_by_id_serialize(
        self,
        thread_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/conversations/v3/conversations/threads/{threadId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_threads_page(
        self,
        after: Annotated[Optional[StrictStr], Field(description="The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="The maximum number of results to display per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Set the sort order of the response. Valid options are `id` (default) and `latestMessageTimestamp` (which requires the `latestMessageTimestampAfter` field to also be set). If you’re filtering threads by `associatedContactId` , you can sort in descending order by prepending - to the sort option (e.g., `-id` or `-latestMessageTimestampAfter` ). Otherwise, results are always returned in ascending order.")] = None,
        inbox_id: Annotated[Optional[List[StrictInt]], Field(description="The ID of the conversations inbox you can optionally include to retrieve the associated messages for. This parameter cannot be used in conjunction with the `associatedContactId` property.")] = None,
        associated_contact_id: Annotated[Optional[StrictInt], Field(description="The ID of a contact you can optionally include to retrieve a filtered list of conversations for. This parameter cannot be used in conjunction with the `inboxId` property.")] = None,
        thread_status: Annotated[Optional[StrictStr], Field(description="The status of the associated conversations to filter by (either `OPEN` or `CLOSED`). This property must be provided if you’re including the `associatedContactId` query parameter.")] = None,
        latest_message_timestamp_after: Annotated[Optional[datetime], Field(description="The minimum `latestMessageTimestamp`. This is required only when sorting by `latestMessageTimestamp`.")] = None,
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived.")] = None,
        association: Annotated[Optional[List[StrictStr]], Field(description="You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response. ")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CollectionResponsePublicThreadForwardPaging:
        """Get threads.


        :param after: The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to display per page.
        :type limit: int
        :param sort: Set the sort order of the response. Valid options are `id` (default) and `latestMessageTimestamp` (which requires the `latestMessageTimestampAfter` field to also be set). If you’re filtering threads by `associatedContactId` , you can sort in descending order by prepending - to the sort option (e.g., `-id` or `-latestMessageTimestampAfter` ). Otherwise, results are always returned in ascending order.
        :type sort: List[str]
        :param inbox_id: The ID of the conversations inbox you can optionally include to retrieve the associated messages for. This parameter cannot be used in conjunction with the `associatedContactId` property.
        :type inbox_id: List[int]
        :param associated_contact_id: The ID of a contact you can optionally include to retrieve a filtered list of conversations for. This parameter cannot be used in conjunction with the `inboxId` property.
        :type associated_contact_id: int
        :param thread_status: The status of the associated conversations to filter by (either `OPEN` or `CLOSED`). This property must be provided if you’re including the `associatedContactId` query parameter.
        :type thread_status: str
        :param latest_message_timestamp_after: The minimum `latestMessageTimestamp`. This is required only when sorting by `latestMessageTimestamp`.
        :type latest_message_timestamp_after: datetime
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param association: You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response.
        :type association: List[str]
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_threads_page_serialize(
            after=after,
            limit=limit,
            sort=sort,
            inbox_id=inbox_id,
            associated_contact_id=associated_contact_id,
            thread_status=thread_status,
            latest_message_timestamp_after=latest_message_timestamp_after,
            archived=archived,
            association=association,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CollectionResponsePublicThreadForwardPaging",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_threads_page_with_http_info(
        self,
        after: Annotated[Optional[StrictStr], Field(description="The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="The maximum number of results to display per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Set the sort order of the response. Valid options are `id` (default) and `latestMessageTimestamp` (which requires the `latestMessageTimestampAfter` field to also be set). If you’re filtering threads by `associatedContactId` , you can sort in descending order by prepending - to the sort option (e.g., `-id` or `-latestMessageTimestampAfter` ). Otherwise, results are always returned in ascending order.")] = None,
        inbox_id: Annotated[Optional[List[StrictInt]], Field(description="The ID of the conversations inbox you can optionally include to retrieve the associated messages for. This parameter cannot be used in conjunction with the `associatedContactId` property.")] = None,
        associated_contact_id: Annotated[Optional[StrictInt], Field(description="The ID of a contact you can optionally include to retrieve a filtered list of conversations for. This parameter cannot be used in conjunction with the `inboxId` property.")] = None,
        thread_status: Annotated[Optional[StrictStr], Field(description="The status of the associated conversations to filter by (either `OPEN` or `CLOSED`). This property must be provided if you’re including the `associatedContactId` query parameter.")] = None,
        latest_message_timestamp_after: Annotated[Optional[datetime], Field(description="The minimum `latestMessageTimestamp`. This is required only when sorting by `latestMessageTimestamp`.")] = None,
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived.")] = None,
        association: Annotated[Optional[List[StrictStr]], Field(description="You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response. ")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CollectionResponsePublicThreadForwardPaging]:
        """Get threads.


        :param after: The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to display per page.
        :type limit: int
        :param sort: Set the sort order of the response. Valid options are `id` (default) and `latestMessageTimestamp` (which requires the `latestMessageTimestampAfter` field to also be set). If you’re filtering threads by `associatedContactId` , you can sort in descending order by prepending - to the sort option (e.g., `-id` or `-latestMessageTimestampAfter` ). Otherwise, results are always returned in ascending order.
        :type sort: List[str]
        :param inbox_id: The ID of the conversations inbox you can optionally include to retrieve the associated messages for. This parameter cannot be used in conjunction with the `associatedContactId` property.
        :type inbox_id: List[int]
        :param associated_contact_id: The ID of a contact you can optionally include to retrieve a filtered list of conversations for. This parameter cannot be used in conjunction with the `inboxId` property.
        :type associated_contact_id: int
        :param thread_status: The status of the associated conversations to filter by (either `OPEN` or `CLOSED`). This property must be provided if you’re including the `associatedContactId` query parameter.
        :type thread_status: str
        :param latest_message_timestamp_after: The minimum `latestMessageTimestamp`. This is required only when sorting by `latestMessageTimestamp`.
        :type latest_message_timestamp_after: datetime
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param association: You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response.
        :type association: List[str]
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_threads_page_serialize(
            after=after,
            limit=limit,
            sort=sort,
            inbox_id=inbox_id,
            associated_contact_id=associated_contact_id,
            thread_status=thread_status,
            latest_message_timestamp_after=latest_message_timestamp_after,
            archived=archived,
            association=association,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CollectionResponsePublicThreadForwardPaging",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_threads_page_without_preload_content(
        self,
        after: Annotated[Optional[StrictStr], Field(description="The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="The maximum number of results to display per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Set the sort order of the response. Valid options are `id` (default) and `latestMessageTimestamp` (which requires the `latestMessageTimestampAfter` field to also be set). If you’re filtering threads by `associatedContactId` , you can sort in descending order by prepending - to the sort option (e.g., `-id` or `-latestMessageTimestampAfter` ). Otherwise, results are always returned in ascending order.")] = None,
        inbox_id: Annotated[Optional[List[StrictInt]], Field(description="The ID of the conversations inbox you can optionally include to retrieve the associated messages for. This parameter cannot be used in conjunction with the `associatedContactId` property.")] = None,
        associated_contact_id: Annotated[Optional[StrictInt], Field(description="The ID of a contact you can optionally include to retrieve a filtered list of conversations for. This parameter cannot be used in conjunction with the `inboxId` property.")] = None,
        thread_status: Annotated[Optional[StrictStr], Field(description="The status of the associated conversations to filter by (either `OPEN` or `CLOSED`). This property must be provided if you’re including the `associatedContactId` query parameter.")] = None,
        latest_message_timestamp_after: Annotated[Optional[datetime], Field(description="The minimum `latestMessageTimestamp`. This is required only when sorting by `latestMessageTimestamp`.")] = None,
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived.")] = None,
        association: Annotated[Optional[List[StrictStr]], Field(description="You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response. ")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get threads.


        :param after: The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to display per page.
        :type limit: int
        :param sort: Set the sort order of the response. Valid options are `id` (default) and `latestMessageTimestamp` (which requires the `latestMessageTimestampAfter` field to also be set). If you’re filtering threads by `associatedContactId` , you can sort in descending order by prepending - to the sort option (e.g., `-id` or `-latestMessageTimestampAfter` ). Otherwise, results are always returned in ascending order.
        :type sort: List[str]
        :param inbox_id: The ID of the conversations inbox you can optionally include to retrieve the associated messages for. This parameter cannot be used in conjunction with the `associatedContactId` property.
        :type inbox_id: List[int]
        :param associated_contact_id: The ID of a contact you can optionally include to retrieve a filtered list of conversations for. This parameter cannot be used in conjunction with the `inboxId` property.
        :type associated_contact_id: int
        :param thread_status: The status of the associated conversations to filter by (either `OPEN` or `CLOSED`). This property must be provided if you’re including the `associatedContactId` query parameter.
        :type thread_status: str
        :param latest_message_timestamp_after: The minimum `latestMessageTimestamp`. This is required only when sorting by `latestMessageTimestamp`.
        :type latest_message_timestamp_after: datetime
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param association: You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response.
        :type association: List[str]
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_threads_page_serialize(
            after=after,
            limit=limit,
            sort=sort,
            inbox_id=inbox_id,
            associated_contact_id=associated_contact_id,
            thread_status=thread_status,
            latest_message_timestamp_after=latest_message_timestamp_after,
            archived=archived,
            association=association,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CollectionResponsePublicThreadForwardPaging",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_threads_page_serialize(
        self,
        after,
        limit,
        sort,
        inbox_id,
        associated_contact_id,
        thread_status,
        latest_message_timestamp_after,
        archived,
        association,
        var_property,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'multi',
            'inboxId': 'multi',
            'association': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if after is not None:

            _query_params.append(('after', after))

        if limit is not None:

            _query_params.append(('limit', limit))

        if sort is not None:

            _query_params.append(('sort', sort))

        if inbox_id is not None:

            _query_params.append(('inboxId', inbox_id))

        if associated_contact_id is not None:

            _query_params.append(('associatedContactId', associated_contact_id))

        if thread_status is not None:

            _query_params.append(('threadStatus', thread_status))

        if latest_message_timestamp_after is not None:
            if isinstance(latest_message_timestamp_after, datetime):
                _query_params.append(
                    (
                        'latestMessageTimestampAfter',
                        latest_message_timestamp_after.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('latestMessageTimestampAfter', latest_message_timestamp_after))

        if archived is not None:

            _query_params.append(('archived', archived))

        if association is not None:

            _query_params.append(('association', association))

        if var_property is not None:

            _query_params.append(('property', var_property))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/conversations/v3/conversations/threads',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_thread_by_id(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived. Default is false.")] = None,
        association: Annotated[Optional[List[StrictStr]], Field(description="You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response. ")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PublicThread:
        """Get a single thread.

        Returns a single thread.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param archived: Whether to return only results that have been archived. Default is false.
        :type archived: bool
        :param association: You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response.
        :type association: List[str]
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_thread_by_id_serialize(
            thread_id=thread_id,
            archived=archived,
            association=association,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicThread",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_thread_by_id_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived. Default is false.")] = None,
        association: Annotated[Optional[List[StrictStr]], Field(description="You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response. ")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PublicThread]:
        """Get a single thread.

        Returns a single thread.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param archived: Whether to return only results that have been archived. Default is false.
        :type archived: bool
        :param association: You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response.
        :type association: List[str]
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_thread_by_id_serialize(
            thread_id=thread_id,
            archived=archived,
            association=association,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicThread",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_thread_by_id_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived. Default is false.")] = None,
        association: Annotated[Optional[List[StrictStr]], Field(description="You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response. ")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get a single thread.

        Returns a single thread.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param archived: Whether to return only results that have been archived. Default is false.
        :type archived: bool
        :param association: You can specify an association type here of `TICKET`. If this is set the response will included a thread associations object and associated ticket id if present. If there are no associations to a ticket with this conversation, then the thread associations object will not be present on the response.
        :type association: List[str]
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_thread_by_id_serialize(
            thread_id=thread_id,
            archived=archived,
            association=association,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicThread",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_thread_by_id_serialize(
        self,
        thread_id,
        archived,
        association,
        var_property,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'association': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        # process the query parameters
        if archived is not None:

            _query_params.append(('archived', archived))

        if association is not None:

            _query_params.append(('association', association))

        if var_property is not None:

            _query_params.append(('property', var_property))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/conversations/v3/conversations/threads/{threadId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_messages_page_by_thread_id(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        after: Annotated[Optional[StrictStr], Field(description="The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="The maximum number of results to display per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sort direction. Valid options are `createdAt` (ascending), and `-createdAt` (descending, default)")] = None,
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived.")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CollectionResponsePublicMessageForwardPaging:
        """Get message history for a thread.


        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param after: The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to display per page.
        :type limit: int
        :param sort: Sort direction. Valid options are `createdAt` (ascending), and `-createdAt` (descending, default)
        :type sort: List[str]
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_messages_page_by_thread_id_serialize(
            thread_id=thread_id,
            after=after,
            limit=limit,
            sort=sort,
            archived=archived,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CollectionResponsePublicMessageForwardPaging",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_messages_page_by_thread_id_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        after: Annotated[Optional[StrictStr], Field(description="The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="The maximum number of results to display per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sort direction. Valid options are `createdAt` (ascending), and `-createdAt` (descending, default)")] = None,
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived.")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CollectionResponsePublicMessageForwardPaging]:
        """Get message history for a thread.


        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param after: The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to display per page.
        :type limit: int
        :param sort: Sort direction. Valid options are `createdAt` (ascending), and `-createdAt` (descending, default)
        :type sort: List[str]
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_messages_page_by_thread_id_serialize(
            thread_id=thread_id,
            after=after,
            limit=limit,
            sort=sort,
            archived=archived,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CollectionResponsePublicMessageForwardPaging",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_messages_page_by_thread_id_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        after: Annotated[Optional[StrictStr], Field(description="The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="The maximum number of results to display per page.")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sort direction. Valid options are `createdAt` (ascending), and `-createdAt` (descending, default)")] = None,
        archived: Annotated[Optional[StrictBool], Field(description="Whether to return only results that have been archived.")] = None,
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get message history for a thread.


        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param after: The paging cursor token of the last successfully read resource will be returned as the `paging.next.after` JSON property of a paged response containing more results.
        :type after: str
        :param limit: The maximum number of results to display per page.
        :type limit: int
        :param sort: Sort direction. Valid options are `createdAt` (ascending), and `-createdAt` (descending, default)
        :type sort: List[str]
        :param archived: Whether to return only results that have been archived.
        :type archived: bool
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_messages_page_by_thread_id_serialize(
            thread_id=thread_id,
            after=after,
            limit=limit,
            sort=sort,
            archived=archived,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CollectionResponsePublicMessageForwardPaging",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_messages_page_by_thread_id_serialize(
        self,
        thread_id,
        after,
        limit,
        sort,
        archived,
        var_property,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        # process the query parameters
        if after is not None:

            _query_params.append(('after', after))

        if limit is not None:

            _query_params.append(('limit', limit))

        if sort is not None:

            _query_params.append(('sort', sort))

        if archived is not None:

            _query_params.append(('archived', archived))

        if var_property is not None:

            _query_params.append(('property', var_property))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/conversations/v3/conversations/threads/{threadId}/messages',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_message_by_thread_and_message_id(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        message_id: Annotated[StrictStr, Field(description="The unique ID of the message.")],
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PublicMessage:
        """Get a single message.


        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param message_id: The unique ID of the message. (required)
        :type message_id: str
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_message_by_thread_and_message_id_serialize(
            thread_id=thread_id,
            message_id=message_id,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicMessage",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_message_by_thread_and_message_id_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        message_id: Annotated[StrictStr, Field(description="The unique ID of the message.")],
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PublicMessage]:
        """Get a single message.


        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param message_id: The unique ID of the message. (required)
        :type message_id: str
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_message_by_thread_and_message_id_serialize(
            thread_id=thread_id,
            message_id=message_id,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicMessage",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_message_by_thread_and_message_id_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        message_id: Annotated[StrictStr, Field(description="The unique ID of the message.")],
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get a single message.


        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param message_id: The unique ID of the message. (required)
        :type message_id: str
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_message_by_thread_and_message_id_serialize(
            thread_id=thread_id,
            message_id=message_id,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicMessage",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_message_by_thread_and_message_id_serialize(
        self,
        thread_id,
        message_id,
        var_property,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        if message_id is not None:
            _path_params['messageId'] = message_id
        # process the query parameters
        if var_property is not None:

            _query_params.append(('property', var_property))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/conversations/v3/conversations/threads/{threadId}/messages/{messageId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_original_content_by_thread_and_message_id(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        message_id: Annotated[StrictStr, Field(description="The unique ID of the message.")],
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PublicMessageContent:
        """Get the original content of a single message.

        Returns the complete original text and rich text bodies of a message. This will be different from the text and rich text in the message itself if the message's `truncationStatus` is anything other than `NOT_TRUNCATED`.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param message_id: The unique ID of the message. (required)
        :type message_id: str
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_original_content_by_thread_and_message_id_serialize(
            thread_id=thread_id,
            message_id=message_id,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicMessageContent",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_original_content_by_thread_and_message_id_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        message_id: Annotated[StrictStr, Field(description="The unique ID of the message.")],
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PublicMessageContent]:
        """Get the original content of a single message.

        Returns the complete original text and rich text bodies of a message. This will be different from the text and rich text in the message itself if the message's `truncationStatus` is anything other than `NOT_TRUNCATED`.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param message_id: The unique ID of the message. (required)
        :type message_id: str
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_original_content_by_thread_and_message_id_serialize(
            thread_id=thread_id,
            message_id=message_id,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicMessageContent",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_original_content_by_thread_and_message_id_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        message_id: Annotated[StrictStr, Field(description="The unique ID of the message.")],
        var_property: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get the original content of a single message.

        Returns the complete original text and rich text bodies of a message. This will be different from the text and rich text in the message itself if the message's `truncationStatus` is anything other than `NOT_TRUNCATED`.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param message_id: The unique ID of the message. (required)
        :type message_id: str
        :param var_property:
        :type var_property: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_original_content_by_thread_and_message_id_serialize(
            thread_id=thread_id,
            message_id=message_id,
            var_property=var_property,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicMessageContent",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_original_content_by_thread_and_message_id_serialize(
        self,
        thread_id,
        message_id,
        var_property,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        if message_id is not None:
            _path_params['messageId'] = message_id
        # process the query parameters
        if var_property is not None:

            _query_params.append(('property', var_property))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/conversations/v3/conversations/threads/{threadId}/messages/{messageId}/original-content',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_thread_by_id(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        public_thread_update_request: PublicThreadUpdateRequest,
        archived: Annotated[Optional[StrictBool], Field(description="Whether the thread to update is archived. Default is false. A thread's status property can not be updated if the thread is archived.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PublicThread:
        """Update a thread.

        Updates a single thread. Either a thread's status can be updated, or the thread can be restored.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param public_thread_update_request: (required)
        :type public_thread_update_request: PublicThreadUpdateRequest
        :param archived: Whether the thread to update is archived. Default is false. A thread's status property can not be updated if the thread is archived.
        :type archived: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_thread_by_id_serialize(
            thread_id=thread_id,
            public_thread_update_request=public_thread_update_request,
            archived=archived,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicThread",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_thread_by_id_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        public_thread_update_request: PublicThreadUpdateRequest,
        archived: Annotated[Optional[StrictBool], Field(description="Whether the thread to update is archived. Default is false. A thread's status property can not be updated if the thread is archived.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PublicThread]:
        """Update a thread.

        Updates a single thread. Either a thread's status can be updated, or the thread can be restored.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param public_thread_update_request: (required)
        :type public_thread_update_request: PublicThreadUpdateRequest
        :param archived: Whether the thread to update is archived. Default is false. A thread's status property can not be updated if the thread is archived.
        :type archived: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_thread_by_id_serialize(
            thread_id=thread_id,
            public_thread_update_request=public_thread_update_request,
            archived=archived,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicThread",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_thread_by_id_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        public_thread_update_request: PublicThreadUpdateRequest,
        archived: Annotated[Optional[StrictBool], Field(description="Whether the thread to update is archived. Default is false. A thread's status property can not be updated if the thread is archived.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update a thread.

        Updates a single thread. Either a thread's status can be updated, or the thread can be restored.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param public_thread_update_request: (required)
        :type public_thread_update_request: PublicThreadUpdateRequest
        :param archived: Whether the thread to update is archived. Default is false. A thread's status property can not be updated if the thread is archived.
        :type archived: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_thread_by_id_serialize(
            thread_id=thread_id,
            public_thread_update_request=public_thread_update_request,
            archived=archived,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PublicThread",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_thread_by_id_serialize(
        self,
        thread_id,
        public_thread_update_request,
        archived,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        # process the query parameters
        if archived is not None:

            _query_params.append(('archived', archived))

        # process the header parameters
        # process the form parameters
        # process the body parameter
        if public_thread_update_request is not None:
            _body_params = public_thread_update_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/conversations/v3/conversations/threads/{threadId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def send_message_to_thread(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        public_message_egg: PublicMessageEgg,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PublicMessage:
        """Send a message to a thread.

        Send a new message on a thread at the current timestamp.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param public_message_egg: (required)
        :type public_message_egg: PublicMessageEgg
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._send_message_to_thread_serialize(
            thread_id=thread_id,
            public_message_egg=public_message_egg,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "PublicMessage",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def send_message_to_thread_with_http_info(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        public_message_egg: PublicMessageEgg,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PublicMessage]:
        """Send a message to a thread.

        Send a new message on a thread at the current timestamp.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param public_message_egg: (required)
        :type public_message_egg: PublicMessageEgg
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._send_message_to_thread_serialize(
            thread_id=thread_id,
            public_message_egg=public_message_egg,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "PublicMessage",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def send_message_to_thread_without_preload_content(
        self,
        thread_id: Annotated[StrictInt, Field(description="The unique ID of the thread.")],
        public_message_egg: PublicMessageEgg,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Send a message to a thread.

        Send a new message on a thread at the current timestamp.

        :param thread_id: The unique ID of the thread. (required)
        :type thread_id: int
        :param public_message_egg: (required)
        :type public_message_egg: PublicMessageEgg
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._send_message_to_thread_serialize(
            thread_id=thread_id,
            public_message_egg=public_message_egg,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "PublicMessage",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _send_message_to_thread_serialize(
        self,
        thread_id,
        public_message_egg,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if thread_id is not None:
            _path_params['threadId'] = thread_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if public_message_egg is not None:
            _body_params = public_message_egg


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json',
                    '*/*'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oauth2_legacy',
            'private_apps_legacy'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/conversations/v3/conversations/threads/{threadId}/messages',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





import functools
from typing import Callable

from .client import Client
from hubspot.discovery.discovery_base import metadata


class OAuthClient(Client):
    """
    OAuthClient extends Client to include automatic token management
    functionality for managing OAuth access tokens with refresh capability.

    Attributes:
        client_id (str): The client ID for OAuth.
        client_secret (str): The client secret for OAuth.
        access_token (str): The access token used for API requests.
        refresh_token (str): The refresh token to obtain a new access token.
        access_token_setter_callback (Callable[[dict], None], optional):
            A callback function that takes new token data as input. Useful for updating the tokens in a database.
    """

    def __init__(self,
                 client_id: str,
                 client_secret: str,
                 access_token: str,
                 refresh_token: str,
                 access_token_setter_callback: Callable[[dict], None] = None,
                 *args,
                 **kwargs
                 ):

        # Function to refresh OAuth tokens
        def _refresh_tokens(config):
            hubspot_client = Client()

            # Perform tokens refresh using HubSpot OAuth API
            tokens = hubspot_client.oauth.tokens_api.create(
                grant_type="refresh_token",
                client_id=config['client_id'],
                client_secret=config['client_secret'],
                refresh_token=config['refresh_token']
            )

            return tokens

        class UnauthorizedException(Exception):
            status = 401

        # Decorator to automatically refresh tokens on unauthorized exceptions
        def _custom_call_api_decorator(config, api_client, call_api):
            @functools.wraps(call_api)
            def wrapper(*args, **kwargs):
                try:
                    response = call_api(*args, **kwargs)
                    if hasattr(response, 'status') and response.status == 401:
                        # openapi-generator >= 7.0.0 does not raise an exception in call_api. We raise it manuall
                        raise UnauthorizedException("The OAuth token used to make this call expired.")
                    return response
                except Exception as e:
                    if hasattr(e, 'status') and e.status == 401:
                        # Refresh token and retry
                        new_tokens = _refresh_tokens(config)
                        config['access_token'] = new_tokens.access_token
                        self.config['access_token'] = new_tokens.access_token
                        api_client.configuration.access_token = new_tokens.access_token
                        if isinstance(e, UnauthorizedException):
                            # if package was generated with openapi-generator >= 7.0.0, modify the header argument
                            args[2]['Authorization'] = f'Bearer {new_tokens.access_token}'
                        if 'access_token_setter_callback' in config:
                            config['access_token_setter_callback'](new_tokens)
                        return call_api(*args, **kwargs)
                    else:
                        raise

            return wrapper

        # Factory function to create API client with auto token refresh
        def _autotokensrefresh_api_factory(api_client_package, api_name, config):
            configuration = api_client_package.Configuration()
            for key in config:
                if key == "retry":
                    configuration.retries = config["retry"]
                else:
                    setattr(configuration, key, config[key])

            api_client = api_client_package.ApiClient(configuration=configuration)
            package_version = metadata.version("hubspot-api-client")
            api_client.user_agent = "hubspot-api-client-python; {0}".format(package_version)

            # Set API call to use custom decorator for automatic token refresh
            api_client.call_api = _custom_call_api_decorator(config, api_client, api_client.call_api)

            return getattr(api_client_package, api_name)(api_client=api_client)

        super().__init__(
            *args,
            **kwargs,
            client_id=client_id,
            client_secret=client_secret,
            access_token=access_token,
            refresh_token=refresh_token,
            access_token_setter_callback=access_token_setter_callback,
            api_factory=_autotokensrefresh_api_factory,
        )



# coding: utf-8

"""
    Hubdb

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from hubspot.cms.hubdb.configuration import Configuration


class ColumnRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"foreign_table_id": "int", "name": "str", "options": "list[Option]", "id": "int", "label": "str", "type": "str", "foreign_column_id": "int"}

    attribute_map = {"foreign_table_id": "foreignTableId", "name": "name", "options": "options", "id": "id", "label": "label", "type": "type", "foreign_column_id": "foreignColumnId"}

    def __init__(self, foreign_table_id=None, name=None, options=None, id=None, label=None, type=None, foreign_column_id=None, local_vars_configuration=None):  # noqa: E501
        """ColumnRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._foreign_table_id = None
        self._name = None
        self._options = None
        self._id = None
        self._label = None
        self._type = None
        self._foreign_column_id = None
        self.discriminator = None

        if foreign_table_id is not None:
            self.foreign_table_id = foreign_table_id
        self.name = name
        self.options = options
        self.id = id
        self.label = label
        self.type = type
        if foreign_column_id is not None:
            self.foreign_column_id = foreign_column_id

    @property
    def foreign_table_id(self):
        """Gets the foreign_table_id of this ColumnRequest.  # noqa: E501

        The id of another table to which the column refers/points to.  # noqa: E501

        :return: The foreign_table_id of this ColumnRequest.  # noqa: E501
        :rtype: int
        """
        return self._foreign_table_id

    @foreign_table_id.setter
    def foreign_table_id(self, foreign_table_id):
        """Sets the foreign_table_id of this ColumnRequest.

        The id of another table to which the column refers/points to.  # noqa: E501

        :param foreign_table_id: The foreign_table_id of this ColumnRequest.  # noqa: E501
        :type foreign_table_id: int
        """

        self._foreign_table_id = foreign_table_id

    @property
    def name(self):
        """Gets the name of this ColumnRequest.  # noqa: E501

        Name of the column  # noqa: E501

        :return: The name of this ColumnRequest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ColumnRequest.

        Name of the column  # noqa: E501

        :param name: The name of this ColumnRequest.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def options(self):
        """Gets the options of this ColumnRequest.  # noqa: E501

        Options to choose for select and multi-select columns  # noqa: E501

        :return: The options of this ColumnRequest.  # noqa: E501
        :rtype: list[Option]
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this ColumnRequest.

        Options to choose for select and multi-select columns  # noqa: E501

        :param options: The options of this ColumnRequest.  # noqa: E501
        :type options: list[Option]
        """
        if self.local_vars_configuration.client_side_validation and options is None:  # noqa: E501
            raise ValueError("Invalid value for `options`, must not be `None`")  # noqa: E501

        self._options = options

    @property
    def id(self):
        """Gets the id of this ColumnRequest.  # noqa: E501

        Column Id  # noqa: E501

        :return: The id of this ColumnRequest.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ColumnRequest.

        Column Id  # noqa: E501

        :param id: The id of this ColumnRequest.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def label(self):
        """Gets the label of this ColumnRequest.  # noqa: E501

        Label of the column  # noqa: E501

        :return: The label of this ColumnRequest.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this ColumnRequest.

        Label of the column  # noqa: E501

        :param label: The label of this ColumnRequest.  # noqa: E501
        :type label: str
        """
        if self.local_vars_configuration.client_side_validation and label is None:  # noqa: E501
            raise ValueError("Invalid value for `label`, must not be `None`")  # noqa: E501

        self._label = label

    @property
    def type(self):
        """Gets the type of this ColumnRequest.  # noqa: E501

        Type of the column  # noqa: E501

        :return: The type of this ColumnRequest.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ColumnRequest.

        Type of the column  # noqa: E501

        :param type: The type of this ColumnRequest.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        allowed_values = [
            "NULL",
            "TEXT",
            "NUMBER",
            "URL",
            "IMAGE",
            "SELECT",
            "MULTISELECT",
            "BOOLEAN",
            "LOCATION",
            "DATE",
            "DATETIME",
            "CURRENCY",
            "RICHTEXT",
            "FOREIGN_ID",
            "VIDEO",
            "CTA",
            "FILE",
            "JSON",
            "COMPOSITE",
            "CODE",
        ]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and type not in allowed_values:  # noqa: E501
            raise ValueError("Invalid value for `type` ({0}), must be one of {1}".format(type, allowed_values))  # noqa: E501

        self._type = type

    @property
    def foreign_column_id(self):
        """Gets the foreign_column_id of this ColumnRequest.  # noqa: E501

        The id of the column from another table to which the column refers/points to.  # noqa: E501

        :return: The foreign_column_id of this ColumnRequest.  # noqa: E501
        :rtype: int
        """
        return self._foreign_column_id

    @foreign_column_id.setter
    def foreign_column_id(self, foreign_column_id):
        """Sets the foreign_column_id of this ColumnRequest.

        The id of the column from another table to which the column refers/points to.  # noqa: E501

        :param foreign_column_id: The foreign_column_id of this ColumnRequest.  # noqa: E501
        :type foreign_column_id: int
        """

        self._foreign_column_id = foreign_column_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(lambda x: convert(x), value))
            elif isinstance(value, dict):
                result[attr] = dict(map(lambda item: (item[0], convert(item[1])), value.items()))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ColumnRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ColumnRequest):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa
"""
    CMS Site Search

    Use these endpoints for searching content on your HubSpot hosted CMS website(s).  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from hubspot.cms.site_search.models.content_search_result import ContentSearchResult
from hubspot.cms.site_search.models.error import Error
from hubspot.cms.site_search.models.error_detail import ErrorDetail
from hubspot.cms.site_search.models.indexed_data import IndexedData
from hubspot.cms.site_search.models.indexed_field import IndexedField
from hubspot.cms.site_search.models.public_search_results import PublicSearchResults



# coding: utf-8

"""
    Public App Crm Cards

    Allows an app to extend the CRM UI by surfacing custom cards in the sidebar of record pages. These cards are defined up-front as part of app configuration, then populated by external data fetch requests when the record page is accessed by a user.  # noqa: E501

    The version of the OpenAPI document: v3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from hubspot.crm.extensions.cards.configuration import Configuration


class CardFetchBodyPatch(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"serverless_function": "str", "card_type": "str", "object_types": "list[CardObjectTypeBody]", "target_url": "str"}

    attribute_map = {"serverless_function": "serverlessFunction", "card_type": "cardType", "object_types": "objectTypes", "target_url": "targetUrl"}

    def __init__(self, serverless_function=None, card_type=None, object_types=None, target_url=None, local_vars_configuration=None):  # noqa: E501
        """CardFetchBodyPatch - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._serverless_function = None
        self._card_type = None
        self._object_types = None
        self._target_url = None
        self.discriminator = None

        if serverless_function is not None:
            self.serverless_function = serverless_function
        if card_type is not None:
            self.card_type = card_type
        self.object_types = object_types
        if target_url is not None:
            self.target_url = target_url

    @property
    def serverless_function(self):
        """Gets the serverless_function of this CardFetchBodyPatch.  # noqa: E501


        :return: The serverless_function of this CardFetchBodyPatch.  # noqa: E501
        :rtype: str
        """
        return self._serverless_function

    @serverless_function.setter
    def serverless_function(self, serverless_function):
        """Sets the serverless_function of this CardFetchBodyPatch.


        :param serverless_function: The serverless_function of this CardFetchBodyPatch.  # noqa: E501
        :type serverless_function: str
        """

        self._serverless_function = serverless_function

    @property
    def card_type(self):
        """Gets the card_type of this CardFetchBodyPatch.  # noqa: E501


        :return: The card_type of this CardFetchBodyPatch.  # noqa: E501
        :rtype: str
        """
        return self._card_type

    @card_type.setter
    def card_type(self, card_type):
        """Sets the card_type of this CardFetchBodyPatch.


        :param card_type: The card_type of this CardFetchBodyPatch.  # noqa: E501
        :type card_type: str
        """
        allowed_values = ["EXTERNAL", "SERVERLESS"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and card_type not in allowed_values:  # noqa: E501
            raise ValueError("Invalid value for `card_type` ({0}), must be one of {1}".format(card_type, allowed_values))  # noqa: E501

        self._card_type = card_type

    @property
    def object_types(self):
        """Gets the object_types of this CardFetchBodyPatch.  # noqa: E501

        An array of CRM object types where this card should be displayed. HubSpot will call your target URL whenever a user visits a record page of the types defined here.  # noqa: E501

        :return: The object_types of this CardFetchBodyPatch.  # noqa: E501
        :rtype: list[CardObjectTypeBody]
        """
        return self._object_types

    @object_types.setter
    def object_types(self, object_types):
        """Sets the object_types of this CardFetchBodyPatch.

        An array of CRM object types where this card should be displayed. HubSpot will call your target URL whenever a user visits a record page of the types defined here.  # noqa: E501

        :param object_types: The object_types of this CardFetchBodyPatch.  # noqa: E501
        :type object_types: list[CardObjectTypeBody]
        """
        if self.local_vars_configuration.client_side_validation and object_types is None:  # noqa: E501
            raise ValueError("Invalid value for `object_types`, must not be `None`")  # noqa: E501

        self._object_types = object_types

    @property
    def target_url(self):
        """Gets the target_url of this CardFetchBodyPatch.  # noqa: E501

        URL to a service endpoint that will respond with details for this card. HubSpot will call this endpoint each time a user visits a CRM record page where this card should be displayed.  # noqa: E501

        :return: The target_url of this CardFetchBodyPatch.  # noqa: E501
        :rtype: str
        """
        return self._target_url

    @target_url.setter
    def target_url(self, target_url):
        """Sets the target_url of this CardFetchBodyPatch.

        URL to a service endpoint that will respond with details for this card. HubSpot will call this endpoint each time a user visits a CRM record page where this card should be displayed.  # noqa: E501

        :param target_url: The target_url of this CardFetchBodyPatch.  # noqa: E501
        :type target_url: str
        """

        self._target_url = target_url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(lambda x: convert(x), value))
            elif isinstance(value, dict):
                result[attr] = dict(map(lambda item: (item[0], convert(item[1])), value.items()))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CardFetchBodyPatch):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CardFetchBodyPatch):
            return True

        return self.to_dict() != other.to_dict()



class AssociationType:
    PRIMARY_CONTACT_TO_COMPANY = 1
    PRIMARY_COMPANY_TO_CONTACT = 2
    DEAL_TO_CONTACT = 3
    CONTACT_TO_DEAL = 4
    PRIMARY_DEAL_TO_COMPANY = 5
    PRIMARY_COMPANY_TO_DEAL = 6
    CONTACT_TO_TICKET = 15
    TICKET_TO_CONTACT = 16
    DEAL_TO_LINE_ITEM = 19
    LINE_ITEM_TO_DEAL = 20
    PRIMARY_COMPANY_TO_TICKET = 25
    PRIMARY_TICKET_TO_COMPANY = 26
    DEAL_TO_TICKET = 27
    TICKET_TO_DEAL = 28
    DEAL_TO_QUOTE = 63
    QUOTE_TO_DEAL = 64
    QUOTE_TO_LINE_ITEM = 67
    LINE_ITEM_TO_QUOTE = 68
    QUOTE_TO_CONTACT = 69
    CONTACT_TO_QUOTE = 70
    QUOTE_TO_COMPANY = 71
    COMPANY_TO_QUOTE = 72
    CONTACT_TO_FEEDBACK_SUBMISSION = 97
    FEEDBACK_SUBMISSION_TO_CONTACT = 98
    TICKET_TO_FEEDBACK_SUBMISSION = 99
    FEEDBACK_SUBMISSION_TO_TICKET = 100
    COMPANY_TO_CALL = 181
    CALL_TO_COMPANY = 182
    COMPANY_TO_EMAIL = 185
    EMAIL_TO_COMPANY = 186
    COMPANY_TO_MEETING = 187
    MEETING_TO_COMPANY = 188
    COMPANY_TO_NOTE = 189
    NOTE_TO_COMPANY = 190
    COMPANY_TO_TASK = 191
    TASK_TO_COMPANY = 192
    CONTACT_TO_CALL = 193
    CALL_TO_CONTACT = 194
    CONTACT_TO_EMAIL = 197
    EMAIL_TO_CONTACT = 198
    CONTACT_TO_MEETING = 199
    MEETING_TO_CONTACT = 200
    CONTACT_TO_NOTE = 201
    NOTE_TO_CONTACT = 202
    CONTACT_TO_TASK = 203
    TASK_TO_CONTACT = 204
    DEAL_TO_CALL = 205
    CALL_TO_DEAL = 206
    DEAL_TO_EMAIL = 209
    EMAIL_TO_DEAL = 210
    DEAL_TO_MEETING = 211
    MEETING_TO_DEAL = 212
    DEAL_TO_NOTE = 213
    NOTE_TO_DEAL = 214
    DEAL_TO_TASK = 215
    TASK_TO_DEAL = 216
    QUOTE_TO_TASK = 217
    TASK_TO_QUOTE = 218
    TICKET_TO_CALL = 219
    CALL_TO_TICKET = 220
    TICKET_TO_EMAIL = 223
    EMAIL_TO_TICKET = 224
    TICKET_TO_MEETING = 225
    MEETING_TO_TICKET = 226
    TICKET_TO_NOTE = 227
    NOTE_TO_TICKET = 228
    TICKET_TO_TASK = 229
    TASK_TO_TICKET = 230
    CONTACT_TO_COMPANY = 279
    COMPANY_TO_CONTACT = 280
    TICKET_TO_COMPANY = 339
    COMPANY_TO_TICKET = 340
    DEAL_TO_COMPANY = 341
    COMPANY_TO_DEAL = 342
    CALL_TO_MEETING = 399
    MEETING_TO_CALL = 400
    POSTAL_MAIL_TO_CONTACT = 453
    CONTACT_TO_POSTAL_MAIL = 454
    POSTAL_MAIL_TO_TICKET = 455
    TICKET_TO_POSTAL_MAIL = 456
    POSTAL_MAIL_TO_DEAL = 457
    DEAL_TO_POSTAL_MAIL = 458
    POSTAL_MAIL_TO_COMPANY = 459
    COMPANY_TO_POSTAL_MAIL = 460
